-- SCRIPT FOR "DEMAGOG" PROGRAM
-- TXT to EPUB Converter
-- (based on original development by tonio_k)
-- PRESS F2


--------------------------------------------------
if CurrentLang() == 'Russian' then
    msgConvert = '...'
    msgCreate = '  ...'
    msgDone = '!'
    msgCap = 'TXT to EPUB Converter'
    msgRU = ' '
    msgEN = ' '
    msgTitle = ''
    msgAuthor = ''
    msgLogLine = '  , 1-2 '
    msgUUID = '  '..string.rep(' ',42)..'UUID' 
    msgHintBt1 = '  ,|, '
    msgCapBt3 = ''
    msgHintBt3 = ' |  '
    msgWorkFolder = '    '
    msgNoFolder = '   !'
    msgNoTemplateFolder = '   !'
    msgUserGuide = '|'..string.rep(' ',28)..'C     EPUB||      txt     . ,       200 .          1 :  .||!|     ,          ! ,    2- : 100.txt -   ; 101.txt -  1; 102.txt -  2; ... 200.txt -   ; 201.txt -  1  ..|     ,               .       .||    ,       _.txt.|    ,       __.txt.||  : jpg, png.       , : cover  title.           . : i_001.jpg, i_002.jpg, i_003.png  ..||   :  2000px,  1400px;   :  1024px,  726px.||       :  <i></i>, <b></b>, <b><i> </i></b>.   ,          ,    .|     ,  #  .|       , : [3].|    ,   . ""        .      ,      (   ): ||!|       /   png.   -   xhtml- .||             .     , ,       :)|| : tonio_k     .'
else
    msgConvert = 'Conversion...'
    msgCreate = 'E-book creation'
    msgDone = 'Done!'    
    msgCap = 'TXT to EPUB Converter'
    msgRU = 'russian'
    msgEN = 'english'
    msgTitle = 'Title'
    msgAuthor = 'Author'
    msgLogLine = 'Brief retelling of the plot (1-2 sentenses)'
    msgUUID = 'Universally unique identifier'..string.rep(' ',72)..'UUID' 
    msgHintBt1 = 'Get it from the link|and copy, paste'
    msgCapBt3 = 'Clear'
    msgHintBt3 = 'Clear form for|new project'
    msgWorkFolder = 'Working folder with project materials'
    msgNoFolder = 'This folder does not exist!'
    msgNoTemplateFolder = 'Template folder not found!'
    msgUserGuide = '|'..string.rep(' ',33)..'CREATING AN ELECTRONIC BOOK IN EPUB FORMAT||The source texts of all chapters in txt format should be in a separate folder. It is desirable that the size of each chapter does not exceed 200 kb. There must be at least 1 illustration in the same folder: book cover.||Important!|The names of the text files should be such that their alphabetical order matches the ORDER of CHAPTERS in the book being created! For example, for a 2-part book: 100.txt - title of the first part; 101.txt - chapter 1; 102.txt - chapter 2; ... 200.txt - title of the second part; 201.txt - chapter 1 etc.|When formatting the text of a chapter, remember that its FIRST line is perceived as a CHAPTER HEADER and will be highlighted in the book in font size. In the rest of the text, the font will be the standard book font.||If the book contains an annotation, then its file should have the name _.txt.|If the book contains an epigraph, then its file should have the name __.txt.||Accepted illustration types: jpg, png. The cover and title page have predefined names, respectively: cover and title. For other illustrations  is desirable to use short unified names with continuous numbering. For example: i_001.jpg, i_002.jpg, i_003.png etc.||Recommended sizes for cover: height 2000px, width 1400px; for other pictures: height 1024px, width 726px.||In the source texts of chapters, it is allowed to specify the font style:  <i>italic</i>, <b>bold</b>, <b><i>bold italic</i></b>. Keep in mind that the corresponding end tag must be in the same paragraph as the start tag.|The location of the illustration is indicated on a separate line as #illustration file name.|References to notes are indicated by numbers in square brackets, for example: [3].|The texts of the notes are formatted in the same way as the texts of the chapters. The "heading" of such a chapter is the note number on the first line. All note files must be preceded by a file containing only one header (no dot at the end): Notes||Important!|Tables in the text should be in the form of b/w drawings in png format. Most reader programs do not understand xhtml table markup.||After creating an e-book, all source materials for it remain intact. And they can be reused, for example, when you want to improve your creation :)|| Acknowledgments: Tonio_K for the idea and initial development.' 
end
--------------------------------------------------


if FolderExists == nil then FolderExists = IsFolder end  -- compatibility with versions < 402


local bom = '\239\187\191'


function Translit(s)
    local x = {}
    x[''] = 'a'
    x[''] = 'b'
    x[''] = 'v'
    x[''] = 'g'
    x[''] = 'd'
    x[''] = 'e'
    x[''] = 'yo'
    x[''] = 'zh'
    x[''] = 'z'
    x[''] = 'i'
    x[''] = 'j'
    x[''] = 'k'
    x[''] = 'l'
    x[''] = 'm'
    x[''] = 'n'
    x[''] = 'o'
    x[''] = 'p'
    x[''] = 'r'
    x[''] = 's'
    x[''] = 't'
    x[''] = 'u'
    x[''] = 'f'
    x[''] = 'h'
    x[''] = 'c'
    x[''] = 'ch'
    x[''] = 'sh'
    x[''] = 'csh'
    x[''] = ''
    x[''] = 'y'
    x[''] = ''
    x[''] = 'e'
    x[''] = 'yu'
    x[''] = 'ya'
    --
    x[''] = 'A'
    x[''] = 'B'
    x[''] = 'V'
    x[''] = 'G'
    x[''] = 'D'
    x[''] = 'E'
    x[''] = 'YO'
    x[''] = 'ZH'
    x[''] = 'Z'
    x[''] = 'I'
    x[''] = 'J'
    x[''] = 'K'
    x[''] = 'L'
    x[''] = 'M'
    x[''] = 'N'
    x[''] = 'O'
    x[''] = 'P'
    x[''] = 'R'
    x[''] = 'S'
    x[''] = 'T'
    x[''] = 'U'
    x[''] = 'F'
    x[''] = 'H'
    x[''] = 'C'
    x[''] = 'CH'
    x[''] = 'SH'
    x[''] = 'CSH'
    x[''] = ''
    x[''] = 'Y'
    x[''] = ''
    x[''] = 'E'
    x[''] = 'YU'
    x[''] = 'YA'
    --
    x[' '] = ' '
    x['.'] = '.'
    x[','] = ','
    --  
    local c, b0, b = '', '', ''
    for i = 1,#s do
        b = s:sub(i,i)
        if i > 1 then b0 = s:sub(i-1,i-1) end
        if (b0 == '' or b0 == ' ' or b0 == '' or b0 == '' ) and (b == '')  then x[b] = 'ja' end
        if (b0 == '' or b0 == ' ' or b0 == '' or b0 == '' ) and (b == '')  then x[b] = 'jo' end
        if (b0 == '' or b0 == ' ' or b0 == '' or b0 == '' ) and (b == '')  then x[b] = 'ju' end
        if (b0 == '' or b0 == ' ' or b0 == '' or b0 == '' ) and (b == '')  then x[b] = 'Ja' end
        if (b0 == '' or b0 == ' ' or b0 == '' or b0 == '' ) and (b == '')  then x[b] = 'Jo' end
        if (b0 == '' or b0 == ' ' or b0 == '' or b0 == '' ) and (b == '')  then x[b] = 'Ju' end
        if x[b] then
            c = c..x[b]
        end
    end
    return c
end


-- get 1st non-empty line of text
function FirstLine(text)
    local a = string.split(text,'\r')
    for i = 1,#a do
        L = string.trim(a[i])
        if L ~= '' then 
            res = L
            break
        end
    end
    return res
end


-- get the table of contents of the book by chapter texts and get the actual chapter texts
-- x - chapter title table
-- y - chapter texts table
-- k - the sequence number of the file from which the "chapters" - notes begin
-- An, Ep - annotation and epigraph texts
function TableOfContents(folder)
    local f = AllFiles(folder, '*.txt')
    local x, y, k, An, Ep = {}, {}, 999999, '', ''
    for i = 1, #f do
        if f[i] ~= '_.txt' and f[i] ~= '__.txt' then
            WOpen(-1,folder..f[i])
            y[#y+1] = WText(-1,true)
            x[#x+1] = Utf8ToAnsi(FirstLine(y[#y]))
            if x[i] == '' or x[i] == 'Notes' then k = i end
        end
    end
    if FileExists(folder..'_.txt') then An = LoadText(folder..'_.txt') end
    if FileExists(folder..'__.txt') then Ep = LoadText(folder..'__.txt') end
    return x, y, k, An, Ep
end


function PicType(PicName)
    local res
    if PicName:sub(#PicName-3,#PicName) == '.jpg' then res = 'jpeg' end
    if PicName:sub(#PicName-3,#PicName) == '.png' then res = 'png' end
    return res
end


-- find links to notes in chapter texts
function LinkProcessing(x, y, k)
    local lnk = [[<a id="n_^D_back" href="section^Q.xhtml#n_^D">[^D]</a>]]
    local back = [[<a class="note_anchor" href="section^I.xhtml#n_^D_back">n_^D</a>]]
    if k > #y then return y end  -- there are no notes in the book
    for i = 1,#y do
        p = 1
        u = k+1
        while true do
            j1, j2 = string.find(y[i],'%[%d+%]',p)
            if j1 == nil then break end
            d = string.sub(y[i],j1+1,j2-1)
            for q = u,#y do
                if x[q] == d then
                    -- match of link [d] in text number i with text number q 
                    --ShowMessage(d..' '..i..' '..q)  --for debugging
                    L = lnk
                    L = L:gsub('%^D',tostring(d))
                    L = L:gsub('%^Q',tostring(q))
                    y[i] = y[i]:sub(1, j1-1)..L..y[i]:sub(j2+1,#y[i])
                    B = back
                    B = B:gsub('%^I',tostring(i))
                    B = B:gsub('%^D',d)
                    y[q] = y[q]..'\r\n'..B
                    u = q+1
                end
            end
            p = j2+#lnk
        end
    end
    return y
end


-- one-line (!) scripts for button actions
A1 = [[Launch('https://www.uuidgenerator.net/','',wsNormal)]]


A2 = [[
if CurrentLang() == 'Russian' then
    msgSource = '    ?'
else
    msgSource = 'Where is the source files folder?'
end
d=Folders(msgSource,Val('Edit7.Text'),true)
if d then
    Let({Edit7 = { Text = d }})
end
]]


A3 = [[
if CurrentLang() == 'Russian' then
    msgClear = ' ,    ?'
else
    msgClear = 'Are you sure you want to clear the form?'
end
if MessageDialog(msgClear) then 
    Let({Edit0 = {Text = ''}})
    Let({Edit1 = {Text = ''}})
    Let({Edit2 = {Text = ''}})
    Let({Edit3 = {Text = ''}})
    Let({Edit6 = {Text = ''}})
    Let({Edit7 = {Text = ''}})
    Let({Memo0 = {Text = ''}})
end
]]


local m = {
Form = { Width = 520, Height = 520, Caption = msgCap },

MarkRU = { Left = 170, Top = 12, Caption = msgRU },
MarkEN = { Left = 350, Top = 12, Caption = msgEN },

Mark0 = { Left = 16, Top = 35, Caption = msgAuthor },
Mark1 = { Left = 16, Top = 63, Caption = msgTitle },
Mark3 = { Left = 16, Top = 95, Caption = msgLogLine },
Mark6 = { Left = 16, Top = 168, Caption = msgUUID },
Mark7 = { Left = 16, Top = 228, Caption = msgWorkFolder },

Edit0 = { Left = 100, Top = 32, Width = 195, Text = '' },
Edit1 = { Left = 305, Top = 32, Width = 195, Text = '' },
Edit2 = { Left = 100, Top = 60, Width = 195, Text = '' },
Edit3 = { Left = 305, Top = 60, Width = 195, Text = '' },
Edit6 = { Left = 270, Top = 184, Width = 230, Text = '' }, 
Edit7 = { Left = 16, Top = 244, Width = 450, Text = '' }, 

Memo0 = { Left = 16, Top = 111, Width = 482, Height = 36, Scroll =true, ReadOnly = false, Text = '' },

Memo1 = { Left = 16, Top = 294, Width = 482, Height = 140, Scroll =true, ReadOnly = true, Text = msgUserGuide},

Button1 = { Caption = 'https://www.uuidgenerator.net/', Left = 16, Top = 184, Width = 230, Height = 20, Action =  A1, Hint = msgHintBt1 },

Button2 = {  Caption = '...', Left = 476, Top = 244, Width = 24, Height = 20, Action = A2 }, 

Button3 = { Caption = msgCapBt3,  Left = 16, Top = 450, Width = 76, Action = A3, Hint = msgHintBt3 },

ButtonOK = { Caption = 'OK', Left = 324, Top = 450, },
ButtonEsc = { Caption = 'Esc', Left = 424, Top = 450 },
}


-- take the data of the previous input, if any
local f, u = HomeFolder()..'e!.@'
if FileExists(f) then
    u = table.load(f)
else
    u = nil
end


local t = Form(m,u)


if t.Answer ~= btOK then goto HALT end


-- save the settings to file
-- excluding from memorization those components and / or properties,
-- which never change when the script is executed
t.Memo1 = nil
table.save(t,f) 


StatusMessage(msgConvert)


local folder =  string.trim(t.Edit7.Text)
if not FolderExists(folder) then 
    ShowMessage(msgNoFolder..'\r'..folder)
    goto HALT
end


ep = HomeFolder('ePub')
if not FolderExists(ep) then
    ShowMessage(msgNoTemplateFolder..'\r'..ep)
    goto HALT
end


local txt_name, txt_table, k, An, Ep = TableOfContents(folder)


if string.trim(t.Edit0.Text) == '' and string.trim(t.Edit1.Text) ~= '' then t.Edit0.Text = t.Edit1.Text end
if string.trim(t.Edit2.Text) == '' and string.trim(t.Edit3.Text) ~= '' then t.Edit2.Text = t.Edit3.Text end
if string.trim(t.Edit1.Text) == '' and string.trim(t.Edit0.Text) ~= '' then t.Edit1.Text = Translit(t.Edit0.Text) end
if string.trim(t.Edit3.Text) == '' and string.trim(t.Edit2.Text) ~= '' then t.Edit3.Text = Translit(t.Edit2.Text) end

local Author = string.trim(t.Edit0.Text)
if Author == '' then Author = string.trim(t.Edit1.Text) end
local Name = string.trim(t.Edit2.Text)
if Name == '' then Name = string.trim(t.Edit3.Text) end
local name = string.trim(t.Edit1.Text)..'_'..string.trim(t.Edit3.Text) -- the name of the book file
if name ~= '_' then
    name = name:gsub('[^%w]','_')
    name = name:gsub('[_]+','_')
else
    name = 'new_book' 
end 
local LogLine = string.trim(t.Memo0.Text)
local UUID = string.trim(t.Edit6.Text)
if UUID == '' then UUID = '00000000-0000-0000-0000-000000000000' end


local txt_image, b = {}, ''
local pic = AllFiles(folder,'*.jpg')
for i = 1,#pic do
    txt_image[#txt_image+1] = pic[i]
    b = b..'copy /b /y "'..folder..pic[i]..'" "'..HomeFolder('ePub\\fish\\OEBPS\\images')..pic[i]..'"\r\n'
end
pic = AllFiles(folder,'*.png')
for i = 1,#pic do
    txt_image[#txt_image+1] = pic[i]
    b = b..'copy /b /y "'..folder..pic[i]..'" "'..HomeFolder('ePub\\fish\\OEBPS\\images')..pic[i]..'"\r\n'
end
SaveToFile({b},'b.bat')
Execute('b.bat','',wsHide)
os.remove('b.bat')


-- cover / title page may be or not
local CovTag, TtlTag, sfn = '', ''
for i = 1,#txt_image do
    sfn = SplitFileName(txt_image[i])
    if sfn[2]:lower() == 'cover' then CovTag = [[<img class="coverimage" alt="Cover" src="images/]]..sfn[2]..sfn[3]..[[" />]] end
    if sfn[2]:lower() == 'title' then TtlTag = [[<img alt="" src="images/]]..sfn[2]..sfn[3]..[[" />]] end
end


-- cover.xhtml
local cov =
[[<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <link href="css/main.css" type="text/css" rel="Stylesheet" />
    <link href="css/default.css" type="text/css" rel="Stylesheet" />
    <title>Cover</title>
  </head>
  <body>
    <div class="coverpage">
]]..CovTag..[[
    </div>
  </body>
</html>]]
cov = cov:gsub('\n','\r\n')
cov = AnsiToUtf8(cov,true)  -- utf8 with BOM!
SaveToFile({cov},HomeFolder('ePub\\fish\\OEBPS')..'cover.xhtml')


-- title.xhtml
local ttl =
[[<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <link href="css/main.css" type="text/css" rel="Stylesheet" />
    <link href="css/default.css" type="text/css" rel="Stylesheet" />
    <title>Title</title>
  </head>
  <body>
    <div class="titlepage">
      <p class="empty-line"> </p>
      <div class="section_image">
]]..TtlTag..[[
      </div>
      <p class="empty-line"> </p>
    </div>
  </body>
</html>]]
ttl = ttl:gsub('\n','\r\n')
ttl = AnsiToUtf8(ttl,true)
SaveToFile({ttl},HomeFolder('ePub\\fish\\OEBPS')..'title.xhtml')


-- annotation.xhtml
An = An:split('\r\n')
for i = 1,#An do
    L = An[i]:trim()
    An[i] = [[<p>]]..L..[[</p>]]      
end
An = table.concat(An,'\r\n')


local anno = 
[[<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <link href="css/main.css" type="text/css" rel="Stylesheet" />
    <link href="css/default.css" type="text/css" rel="Stylesheet" />
    <title>Annotation</title>
  </head>
  <body>
    <div class="annotation">
]]..An..[[
    </div>
  </body>
</html>
]]
anno = anno:gsub('\n','\r\n')
anno = AnsiToUtf8(anno,true)
SaveToFile({anno},HomeFolder('ePub\\fish\\OEBPS')..'annotation.xhtml')


-- epigraph.xhtml
Ep = Ep:split('\r\n')
for i = 1,#Ep do
    L = Ep[i]:trim()
    Ep[i] = [[<p><i>]]..L..[[</i></p>]]      
end
Ep = table.concat(Ep,'\r\n')


local epi =
[[<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <link href="css/main.css" type="text/css" rel="Stylesheet" />
    <link href="css/default.css" type="text/css" rel="Stylesheet" />
    <title></title>
  </head>
  <body>
    <div class="epigraph">
]]..Ep..[[
    </div>
  </body>
</html>
]]
epi = epi:gsub('\n','\r\n')
epi = AnsiToUtf8(epi,true)
SaveToFile({epi},HomeFolder('ePub\\fish\\OEBPS')..'epigraph.xhtml')


-- about.xhtml
local abo =
[[<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <link href="css/main.css" type="text/css" rel="Stylesheet" />
    <link href="css/default.css" type="text/css" rel="Stylesheet" />
    <title>About</title>
  </head>
  <body>
    <div class="about">
      <h1>About</h1>
      <p>This e-book was created using the Demagog computer program.</p>
    </div>
  </body>
</html>]]
abo = abo:gsub('\n','\r\n')
abo = AnsiToUtf8(abo,true)
SaveToFile({abo},HomeFolder('ePub\\fish\\OEBPS')..'about.xhtml')


-- Content.opf
local opf0 = {}    
local opf1 = {}
local opf2 = {}
local opf3 = {}
local ncx = {}
for i = 1,#txt_image do
    opf0[#opf0+1] = [[<item id="]]..txt_image[i]..[[" href="images/]]..txt_image[i]..[[" media-type="image/]]..PicType(txt_image[i])..[["/>]]
end
for i = 1,#txt_table do
    opf1[#opf1+1] = [[<item id="section]]..i..[[" href="section]]..i..[[.xhtml" media-type="application/xhtml+xml"/>]]
    opf2[#opf2+1] = [[<itemref idref="section]]..i..[["/>]]
    if i <= k then 
        opf3[#opf3+1] = [[<reference type="text" title="section]]..i..[[" href="section]]..i..[[.xhtml" />]]
    else
        opf3[#opf3+1] = [[<reference type="glossary" title="section]]..i..[[" href="section]]..i..[[.xhtml" />]]
    end
    ncx[#ncx+1] = [[<navPoint id="section]]..i..[[" playOrder="]]..i..[["><navLabel><text>]]..txt_name[i]..[[</text></navLabel><content src="section]]..i..[[.xhtml"/></navPoint>]]
end
--  
opf0 = table.concat(opf0, '\r\n')
opf1 = table.concat(opf1, '\r\n') 
opf2 = table.concat(opf2, '\r\n') 
opf3 = table.concat(opf3, '\r\n') 
ncx = table.concat(ncx, '\r\n') 


fb_opf = [[<?xml version="1.0" encoding="UTF-8"?>
<package version="2.0" xmlns="http://www.idpf.org/2007/opf"
    unique-identifier="BookID">
        <metadata xmlns:dc="http://purl.org/dc/elements/1.1/"
        xmlns:opf="http://www.idpf.org/2007/opf">
            <dc:title>]]..Name..[[</dc:title>
            <dc:language>ru</dc:language>
            <dc:identifier id="BookID" opf:scheme="URI">urn:uuid:]]..UUID..[[</dc:identifier>
            <dc:identifier id="BookISBN" opf:scheme="ISBN">0-000-00000</dc:identifier>
            <dc:creator opf:role="aut">]]..Author..[[</dc:creator>
            <dc:description>]]..LogLine..[[
</dc:description>
</metadata>
    <manifest>
]]..opf0..[[   
<item id="LiberationSerif-Regular" href="fonts/LiberationSerif-Regular.ttf" media-type="application/x-font-ttf" />
<item id="LiberationSerif-Italic" href="fonts/LiberationSerif-Italic.ttf" media-type="application/x-font-ttf" />
<item id="mainCSS" href="css/main.css" media-type="text/css" />
<item id="default" href="css/default.css" media-type="text/css" />
<item id="cover" href="cover.xhtml" media-type="application/xhtml+xml" />
<item id="annotation" href="annotation.xhtml" media-type="application/xhtml+xml" />
<item id="epigraph" href="epigraph.xhtml" media-type="application/xhtml+xml" />
<item id="title" href="title.xhtml" media-type="application/xhtml+xml" />
<item id="ncx" href="toc.ncx" media-type="application/x-dtbncx+xml" />
]]..opf1..[[

<item id="about" href="about.xhtml" media-type="application/xhtml+xml" />
    </manifest>
    <spine toc="ncx">
<itemref idref="cover"/>
<itemref idref="title"/>
<itemref idref="annotation"/>
<itemref idref="epigraph"/>
]]..opf2..[[

<itemref idref="about"/>
    </spine>
    <guide>
<reference type="cover" title="cover" href="cover.xhtml" />
<reference type="title-page" title="title" href="title.xhtml" />
<reference type="preface" title="annotation" href="annotation.xhtml" /> 
]]..opf3..[[

<reference type="copyright-page" title="about" href="about.xhtml" /> 
    </guide>
</package>
]]
fb_opf = fb_opf:gsub('\n','\r\n')
fb_opf = AnsiToUtf8(fb_opf,true)


-- toc.ncx
fb_ncx = [[<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<ncx xmlns="http://www.daisy.org/z3986/2005/ncx/" version="2005-1" xml:lang="ru-RU">
    <head>
        <meta name="dtb:uid" content="urn:uuid:]]..UUID..[["/>
        <meta name="dtb:depth" content="1"/>
        <meta name="dtb:totalPageCount" content="0"/>
        <meta name="dtb:maxPageNumber" content="0"/>
    </head>
    <docTitle><text>]]..name..[[</text>
    </docTitle>
    <docAuthor>
        <text>]]..Author..[[</text>
    </docAuthor>
    <navMap>
]]..ncx..[[
    </navMap>
</ncx>
]]
fb_ncx = fb_ncx:gsub('\n','\r\n')
fb_ncx = AnsiToUtf8(fb_ncx)
SaveToFile({fb_opf},HomeFolder('ePub\\fish\\OEBPS')..'Content.opf')
SaveToFile({bom..fb_ncx},HomeFolder('ePub\\fish\\OEBPS')..'toc.ncx')


-- search links
txt_table = LinkProcessing( txt_name,  txt_table, k)


local img1 = [[<p><img class="int_image" alt="" src="images/]]
local img2 = [["/> </p>]]
for i = 1,#txt_table do
    txt_table[i] = txt_table[i]:gsub('\r\n','\r')
    txt_table[i] = txt_table[i]:gsub('\n','\r')
    txt_table[i] = StrSplit(txt_table[i],'\r')
    txt_table[i][1] = [[<div class="title2"><h2>]].. txt_table[i][1]..[[</h2></div>]]
    for j = 2,#txt_table[i] do
        L = txt_table[i][j]:trim()
        if L ~= '' then
            if L:sub(1,1) == '#' then
                txt_table[i][j] = L:gsub('#',img1)..img2
            else
                txt_table[i][j] = [[<p>]]..L..[[</p>]]
            end          
        else
            txt_table[i][j] = '<p class="empty-line"></p>'
        end
    end
    txt_table[i] = table.concat(txt_table[i], '\r\n') --  
end


local h0 = {}
local n = math.min(k,#txt_table)
for i = 1, n do
    h0[#h0+1] = 
[[<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <link href="css/main.css" type="text/css" rel="Stylesheet" />
    <link href="css/default.css" type="text/css" rel="Stylesheet" />
    <title>]]..txt_name[i]..[[</title>
  </head>
  <body>
    <div class="section1">]]
    h0[i] = h0[i]:gsub('\n','\r\n')
    h0[i] = AnsiToUtf8(h0[i])   -- utf8 no BOM (2nd parameter false by default)
end


local hn0 = {}
if n < #txt_table then
    for i = n+1, #txt_table do
hn0[#hn0+1] = 
[[<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <link href="css/main.css" type="text/css" rel="Stylesheet" />
    <link href="css/default.css" type="text/css" rel="Stylesheet" />
    <title>]]..txt_name[i]..[[</title>
  </head>
  <body>
    <div class="section1" id="n_]]..txt_name[i]..[[">]]
        hn0[#hn0] = hn0[#hn0]:gsub('\n','\r\n')
        hn0[#hn0] = AnsiToUtf8(hn0[#hn0])
    end
end


local h1 = 
[[

    </div>
  </body>
</html>]]
h1 = AnsiToUtf8(h1)
h1 = h1:gsub('\n','\r\n')


-- txt_table, h0, hn0, h1 - txt_table, h0, hn0, h1 originally utf8 encoded without BOM
for i = 1,n do
    SaveToFile({bom..h0[i]..txt_table[i]..h1},HomeFolder('ePub\\fish\\OEBPS')..[[section]]..i..[[.xhtml]])
end
if n < #txt_table then
    for i = n+1,#txt_table do
        SaveToFile({bom..hn0[i-n]..txt_table[i]..h1},HomeFolder('ePub\\fish\\OEBPS')..[[section]]..i..[[.xhtml]])
    end
end


StatusMessage(msgCreate)


local dos = {}
os.remove(folder..name..'.epub')  -- delete to create a new one, not update the archive
dos[#dos+1] = AnsiToDos('copy "'..HomeFolder('ePub')..'Sample.zip" "'..folder..name..'.epub"')
dos[#dos+1] = AnsiToDos('"'..HomeFolder('ePub')..'7z.exe" u -tzip -mx9 -r0 "'..folder..name..'.epub" "'..HomeFolder('ePub\\fish')..'*"')
dos[#dos+1] = AnsiToDos('"'..HomeFolder('ePub')..'7z.exe" rn  "'..folder..name..'.epub" 1mimetype mimetype')
dos[#dos+1] = 'del %0'  -- the batch file will delete itself
SaveToFile(dos,'bat.bat')
Execute('bat.bat','',wsHide)


-- deleting temporary files
os.remove(HomeFolder('ePub\\fish\\OEBPS')..'Content.opf')
os.remove(HomeFolder('ePub\\fish\\OEBPS')..'toc.ncx')
os.remove(HomeFolder('ePub\\fish\\OEBPS')..'cover.xhtml')
os.remove(HomeFolder('ePub\\fish\\OEBPS')..'title.xhtml')
os.remove(HomeFolder('ePub\\fish\\OEBPS')..'annotation.xhtml')
os.remove(HomeFolder('ePub\\fish\\OEBPS')..'epigraph.xhtml')
os.remove(HomeFolder('ePub\\fish\\OEBPS')..'about.xhtml')
for i = 1,#txt_name do
     os.remove(HomeFolder('ePub\\fish\\OEBPS')..[[section]]..i..[[.xhtml]])
end
for i = 1,#txt_image do
     os.remove(HomeFolder('ePub\\fish\\OEBPS\\images')..txt_image[i])
end


StatusMessage(msgDone..' '..name)
pause(2)
StatusMessage('')


::HALT::
